require 'hakuban'
require_relative './cpu-usage.rb'



class UtilizationManager < Hakuban::ObjectManager
	def object_insert(object)
		object.id = object.descriptor.json["id"]
	end
end


id = "#{$$}@#{`hostname`.strip}"

hakuban = Hakuban::LocalNode.new
connector = Hakuban::WebsocketConnector.new(hakuban, "ws://localhost:3001")
all_utilizations = hakuban.tag("utilizations").observe.manage(UtilizationManager.new)
my_utilization = hakuban.object(["utilizations"],{id: id}).expose.manage(UtilizationManager.new)

previous_cpu_usage_state = nil
loop {
	previous_cpu_usage_state, cpu_usage = CPU::current_usage(previous_cpu_usage_state)
	my_utilization.data = {"percentage"=>cpu_usage}  if cpu_usage

	utilizations = all_utilizations.values.sort_by(&:id).map { |utilization|
		[utilization.id, utilization.data&.dig("percentage")]
	}.select { |id, utilization| utilization }
	next sleep 1 if utilizations.empty?

	widest_id_width = utilizations.transpose[0].map(&:size).max
	utilizations.each.with_index { |(id, utilization), i|
		bars = utilization.map { |percentage| " ▁▂▃▄▅▆▇█"[percentage*9/100] }.join("")
		bg_color = [0,0,180 + (i % 2)*64]
		fg_color = [255,255,0]
		puts "%s -> \e[48;2;%i;%i;%im\e[38;2;%i;%i;%im%s\e[49m\e[39m"%([id.ljust(widest_id_width)]+bg_color+fg_color+[bars])
	}

	sleep 1
	#TODO: make this more friendly, save state, etc.
	print "\e[2J\e[f"
}